#!/usr/bin/env fades

"""Fixes contacts phone numbers.

Tutorial/docs: https://developers.google.com/people

fades:
    httplib2
    oauth2client
    google-api-python-client
"""

import httplib2
from googleapiclient.discovery import build
from oauth2client.file import Storage
from oauth2client.client import OAuth2WebServerFlow
from oauth2client.tools import run_flow

# need to create a project in
# https://console.developers.google.com/start/api?id=people.googleapis.com&credential=client_key
CLIENT_ID =
CLIENT_SECRET =

# different scopes (need to use one of these below)
SCOPE_READ_ONLY = 'https://www.googleapis.com/auth/contacts.readonly'
SCOPE_READ_WRITE = 'https://www.google.com/m8/feeds/'

USER_AGENT = 'ContactsFixer/1'

VALID_DIGITS = set("0123456789 +")
MOBILE = 'mobile'


def _auth():
    """Complicated oauth stuff."""
    storage = Storage('auth_info.dat')
    credentials = storage.get()
    if credentials is None or credentials.invalid:
        # do auth dance if current credentials are missing or old
        auth_flow = OAuth2WebServerFlow(
            client_id=CLIENT_ID, client_secret=CLIENT_SECRET,
            scope=SCOPE_READ_WRITE, user_agent=USER_AGENT)
        credentials = run_flow(auth_flow, storage)

    # build the service
    http = httplib2.Http()
    http = credentials.authorize(http)
    service = build(serviceName='people', version='v1', http=http)
    return service


def process(old_number, ntype):
    """Process the phone number.

    NOTE: this is by no means complete at all, use at your own risk.
    """
    # remove non digits, cleanup
    old_number = old_number.replace('-', ' ').replace('.', ' ').strip()
    old_number = ''.join(d for d in old_number if d in VALID_DIGITS)

    no_spaced = old_number.replace(' ', '')
    if no_spaced.startswith(("0800", "0810")):
        # not touching these
        return

    if old_number.startswith('+'):
        # already international (returning it anyway because of cleaning)
        return old_number

    # handle most common cases: '11' area prefix
    n_block, n_rest = None, None
    if len(no_spaced) == 8 and old_number.startswith(('4', '5', '6')):
        # short version
        n_block, n_rest = no_spaced[:4], no_spaced[4:]
    elif len(no_spaced) == 10 and old_number.startswith('11'):
        # long version
        n_block, n_rest = no_spaced[2:6], no_spaced[6:]
    elif len(no_spaced) == 11 and old_number.startswith('011'):
        # long version with dialing 0
        n_block, n_rest = no_spaced[3:7], no_spaced[7:]
    elif len(no_spaced) == 10 and old_number.startswith('15') and ntype == MOBILE:
        # short version with dialing 15
        n_block, n_rest = no_spaced[2:6], no_spaced[6:]
    if n_block and n_rest:
        if ntype == MOBILE:
            return "+54 9 11 {} {}".format(n_block, n_rest)
        else:
            return "+54 11 {} {}".format(n_block, n_rest)

    if len(no_spaced) == 10 and old_number.startswith(('2', '3')):
        # long version, national
        if ntype == MOBILE:
            return "+54 9 " + old_number
        else:
            return "+54 " + old_number

    if len(no_spaced) == 11 and old_number.startswith(('02', '03')):
        # long version, national, with dialing 0
        if ntype == MOBILE:
            return "+54 9 " + old_number[1:]
        else:
            return "+54 " + old_number[1:]

    raise ValueError("Unknown number format: {!r} ({!r})".format(old_number, ntype))


# auth
service = _auth()

# get people (me) connections (page size way bigger than my contacts quantity, so I don't need to
# implement pagination)
page_size = 2000
request = service.people().connections().list(
    resourceName='people/me', pageSize=page_size, personFields='names,phoneNumbers')
response = request.execute()
connections = response['connections']
assert len(connections) < page_size  # otherwise we need to implement paging!

for conn in connections:
    resource_name = conn['resourceName']

    names = conn.get('names')
    if not names:
        raise ValueError("No names! " + repr(conn))

    name = names[0]  # no matter if many, this is just as a reference for UI
    print(name['displayName'])

    phone_numbers = conn.get('phoneNumbers')
    if phone_numbers is None:
        print("    (no phone)")
        continue

    modified = False
    for pn in phone_numbers:
        old_value = pn['value']
        number_type = pn['type']
        new_value = process(old_value, number_type)
        if new_value is None or new_value == old_value:
            print("    {!r} {} UNCHANGED".format(old_value, number_type))
            continue

        # different!
        print("    {!r} {} --> {!r}".format(old_value, number_type, new_value))
        modified = True
        pn['value'] = new_value

    if modified:
        # fix contact number
        print("    saving new data in the cloud")
        request = service.people().updateContact(
            resourceName=conn['resourceName'], updatePersonFields='phoneNumbers',
            body=conn)
        request.execute()

print("Done")
